"use strict";

var fse = _interopRequireWildcard(require("fs-extra"));

var _globPromise = _interopRequireDefault(require("glob-promise"));

var path = _interopRequireWildcard(require("path"));

var _pixelmatch = _interopRequireDefault(require("pixelmatch"));

var _sharp = _interopRequireDefault(require("sharp"));

var tmp = _interopRequireWildcard(require("tmp"));

var index = _interopRequireWildcard(require("../index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

describe("index", () => {
  const fixturesPath = path.join(__dirname, "fixtures");
  let tmpDir;
  beforeEach(() => {
    tmpDir = tmp.dirSync({
      unsafeCleanup: true
    });
  });
  afterEach(() => {
    tmpDir.removeCallback();
  });
  it("generates files from example matching fixtures", async () => {
    await testFixture("example", 0.16);
  });
  it("generates files from white matching fixtures", async () => {
    await testFixture("white", 0.1);
  });
  it("generates files from text matching fixtures", () => testFixture("text", 0.14), 20 * 1000);

  async function testFixture(fixture, threshold) {
    const fixtureDir = path.join(fixturesPath, fixture);
    const generator = index.generate({
      icon: path.join(fixtureDir, "icon.svg"),
      resDirPath: path.join(tmpDir.name, "android", "app", "src", "main", "res"),
      iconsetDir: path.join(tmpDir.name, "ios", fixture, "Images.xcassets")
    });
    const generatedFiles = [];

    for await (let file of generator) {
      const localPath = path.relative(tmpDir.name, file);
      const fixturePath = path.join(fixtureDir, localPath);
      await expectFilesToEqual(file, fixturePath, threshold);
      generatedFiles.push(localPath);
    }

    expect(generatedFiles.sort()).toEqual((await (0, _globPromise.default)("*/**", {
      cwd: fixtureDir,
      nodir: true
    })).sort());
  }
});

async function expectFilesToEqual(expected, actual, threshold) {
  expectFileToExist(expected);
  expectFileToExist(actual);
  const extension = path.extname(expected);

  switch (extension) {
    case ".png":
      return expectImagesToEqual(expected, actual, threshold);

    case ".json":
      return expectJsonToEqual(expected, actual);

    case ".xml":
      return expectXmlToEqual(expected, actual);

    default:
      return fail(`Unsupported comparison file type ${extension}`);
  }
}

async function expectFileToExist(path) {
  if (!(await fse.pathExists(path))) {
    fail(`File did not exist: ${path}`);
  }
}

async function expectXmlToEqual(expected, actual) {
  // TODO: Compare XML structure
  expect((await fse.readFile(expected, "utf-8"))).toEqual((await fse.readFile(actual, "utf-8")));
}

async function expectJsonToEqual(expected, actual) {
  expect((await fse.readJson(expected))).toEqual((await fse.readJson(actual)));
}

async function expectImagesToEqual(expected, actual, threshold) {
  const expectedImage = (0, _sharp.default)(expected);
  const actualImage = (0, _sharp.default)(actual);
  const expectedMetadata = await expectedImage.metadata();
  const actualMetadata = await actualImage.metadata();
  expect({
    width: expectedMetadata.width,
    height: expectedMetadata.height
  }).toEqual({
    width: actualMetadata.width,
    height: actualMetadata.height
  });

  if (!(expectedMetadata.width && expectedMetadata.height)) {
    throw new Error(`Image dimensions not match: expected ${expectedMetadata.width}x${expectedMetadata.height}, got ${actualMetadata.width}x${actualMetadata.height}`);
  }

  const totalPixelCount = expectedMetadata.width * expectedMetadata.height;
  const mismatchingPixelCount = (0, _pixelmatch.default)((await expectedImage.toBuffer()), (await actualImage.toBuffer()), null, expectedMetadata.width, expectedMetadata.height, {
    threshold
  });
  expect(mismatchingPixelCount / totalPixelCount).toBeLessThanOrEqual(threshold);
}