"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generate = generate;

var path = _interopRequireWildcard(require("path"));

var _svg2vectordrawable = _interopRequireDefault(require("svg2vectordrawable"));

var input = _interopRequireWildcard(require("./input"));

var output = _interopRequireWildcard(require("./output"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

const adaptiveIconMinSdk = 26;
const densities = [{
  name: "mdpi",
  scale: 1
}, {
  name: "hdpi",
  scale: 1.5
}, {
  name: "xhdpi",
  scale: 2
}, {
  name: "xxhdpi",
  scale: 3
}, {
  name: "xxxhdpi",
  scale: 4
}];
const launcherName = "ic_launcher";
const roundIconName = "ic_launcher_round";
const launcherForegroundName = "ic_launcher_foreground";
/** Adaptive Icon **/

const adaptiveIconBaseSize = 108;

const adaptiveIconContent = launcherForegroundType => `<?xml version="1.0" encoding="utf-8"?>
<adaptive-icon xmlns:android="http://schemas.android.com/apk/res/android">
    <background android:drawable="@android:color/white" />
    <foreground android:drawable="@${launcherForegroundType}/${launcherForegroundName}" />
</adaptive-icon>`;
/** Legacy Icon **/


const legacyIconBaseSize = 48;
const inputIconContentRatio = input.inputContentSize / input.inputImageSize; // Based on images from image asset studio at
// https://android.googlesource.com/platform/tools/adt/idea/+/refs/heads/mirror-goog-studio-master-dev/android/resources/images/launcher_stencil/
// https://android.googlesource.com/platform/tools/adt/idea/+/refs/heads/mirror-goog-studio-master-dev/android/src/com/android/tools/idea/npw/assetstudio/LauncherLegacyIconGenerator.java

const legacyLightningFilter = `
  <filter id="legacyLightningFilter">
    <!-- Drop shadow -->
    <feGaussianBlur in="SourceAlpha" stdDeviation="0.4" />
    <feOffset dx="0" dy="1.125" />
    <feComponentTransfer>
      <feFuncA type="linear" slope="0.2"/>
    </feComponentTransfer>
    <feComposite in2="SourceAlpha" operator="out"
      result="shadow"
    />

    <!-- Edge shade -->
    <feComponentTransfer in="SourceAlpha" result="opaque-alpha">
      <feFuncA type="linear" slope="0.2"/>
    </feComponentTransfer>
    <feOffset dx="-0.2" dy="-0.2" in="SourceAlpha" result="offset-alpha" />
    <feComposite in="opaque-alpha" in2="offset-alpha" operator="out"
      result="edge"
    />

    <feMerge>
      <feMergeNode in="shadow" />
      <feMergeNode in="edge" />
    </feMerge>
  </filter>`;
/** Legacy Square Icon **/

const legacySquareIconContentSize = 38;
const legacySquareIconBorderRadius = 3;
const legacySquareIconMargin = (legacyIconBaseSize - legacySquareIconContentSize) / 2;
const legacySquareIconContentRatio = legacySquareIconContentSize / legacyIconBaseSize;
const legacySquareIconMask = `<svg version="1.1" xmlns="http://www.w3.org/2000/svg"
  viewBox="${getViewBox(legacySquareIconContentSize)}"
  width="${input.inputImageSize}" height="${input.inputImageSize}">
    <rect
      x="${legacySquareIconMargin}" y="${legacySquareIconMargin}"
      width="${legacySquareIconContentSize}" height="${legacySquareIconContentSize}"
      rx="${legacySquareIconBorderRadius}" ry="${legacySquareIconBorderRadius}"
    />
</svg>`;
const legacySquareIconOverlay = `<svg version="1.1" xmlns="http://www.w3.org/2000/svg"
  viewBox="${getViewBox(legacySquareIconContentSize)}"
  width="${input.inputImageSize}" height="${input.inputImageSize}">
    ${legacyLightningFilter}
    <rect
      x="${legacySquareIconMargin}" y="${legacySquareIconMargin}"
      width="${legacySquareIconContentSize}" height="${legacySquareIconContentSize}"
      rx="${legacySquareIconBorderRadius}" ry="${legacySquareIconBorderRadius}"
      filter="url(#legacyLightningFilter)"
    />
</svg>`;
/** Legacy Round Icon **/

const legacyRoundIconContentSize = 44;
const legacyRoundIconContentRatio = legacyRoundIconContentSize / legacyIconBaseSize;
const roundIconMask = `<svg version="1.1" xmlns="http://www.w3.org/2000/svg"
  viewBox="${getViewBox(legacyRoundIconContentSize)}"
  width="${input.inputImageSize}" height="${input.inputImageSize}">
    <circle
      cx="${legacyIconBaseSize / 2}" cy="${legacyIconBaseSize / 2}"
      r="${legacyRoundIconContentSize / 2}"
    />
</svg>`;
const roundIconOverlay = `<svg version="1.1" xmlns="http://www.w3.org/2000/svg"
  viewBox="${getViewBox(legacyRoundIconContentSize)}"
  width="${input.inputImageSize}" height="${input.inputImageSize}">
    ${legacyLightningFilter}
    <circle
      cx="${legacyIconBaseSize / 2}" cy="${legacyIconBaseSize / 2}"
      r="${legacyRoundIconContentSize / 2}"
      filter="url(#legacyLightningFilter)"
    />
</svg>`;

function getViewBox(input) {
  const size = input / inputIconContentRatio;
  const margin = (size - legacyIconBaseSize) / 2;
  const viewBox = [-margin, -margin, size, size];
  return viewBox.join(" ");
}

async function* generate(config, fileInput) {
  const fullConfig = await getConfig(config);
  yield* generateLegacyIcons(fileInput, fullConfig);
  yield* generateRoundIcons(fileInput, fullConfig);
  yield* generateAdaptiveIcon(fileInput, fullConfig);
}

async function getConfig(config) {
  return {
    resDirPath: config.resDirPath || "./android/app/src/main/res",
    vectorDrawables: config.vectorDrawables === undefined ? true : config.vectorDrawables
  };
}

async function* generateLegacyIcons(fileInput, config) {
  yield* output.genaratePngs({ ...fileInput,
    mask: legacySquareIconMask,
    overlay: legacySquareIconOverlay,
    cropSize: input.inputContentSize / legacySquareIconContentRatio
  }, densities.map(density => ({
    filePath: getIconPath(config, "mipmap", {
      density: density.name
    }, `${launcherName}.png`),
    outputSize: legacyIconBaseSize * density.scale
  })));
}

async function* generateRoundIcons(fileInput, config) {
  yield* output.genaratePngs({ ...fileInput,
    mask: roundIconMask,
    overlay: roundIconOverlay,
    cropSize: input.inputContentSize / legacyRoundIconContentRatio
  }, densities.map(density => ({
    filePath: getIconPath(config, "mipmap", {
      density: density.name
    }, `${roundIconName}.png`),
    outputSize: legacyIconBaseSize * density.scale
  })));
}

async function* generateAdaptiveIcon(fileInput, config) {
  let foregroundResourceType;

  try {
    yield* generateAdaptiveVdForeground(fileInput, config);
    foregroundResourceType = "drawable";
  } catch {
    yield* generateAdaptivePngForeground(fileInput, config);
    foregroundResourceType = "mipmap";
  } // Adaptive icon


  const adaptiveIconXml = adaptiveIconContent(foregroundResourceType);
  yield* output.ensureFileContents(getIconPath(config, "mipmap", {
    density: "anydpi",
    minApiLevel: 26
  }, `${launcherName}.xml`), adaptiveIconXml);
  yield* output.ensureFileContents(getIconPath(config, "mipmap", {
    density: "anydpi",
    minApiLevel: 26
  }, `${roundIconName}.xml`), adaptiveIconXml);
}

async function* generateAdaptiveVdForeground(fileInput, config) {
  if (!config.vectorDrawables) {
    throw Error("Vector drawables disabled");
  }

  const fileData = await fileInput.read();
  const vdData = await (0, _svg2vectordrawable.default)(fileData.imageData.data.toString("utf-8"), undefined, true);
  yield* output.ensureFileContents(getIconPath(config, "drawable", {
    density: "anydpi",
    minApiLevel: 26
  }, `${launcherForegroundName}.xml`), vdData);
}

async function* generateAdaptivePngForeground(fileInput, config) {
  yield* output.genaratePngs(fileInput, densities.map(density => ({
    filePath: getIconPath(config, "mipmap", {
      density: density.name,
      minApiLevel: adaptiveIconMinSdk
    }, `${launcherForegroundName}.png`),
    outputSize: adaptiveIconBaseSize * density.scale
  })));
}

function getIconPath(config, resourceType, qualifier, fileName) {
  let directoryName = [resourceType];

  if (qualifier.density) {
    directoryName = [...directoryName, qualifier.density];
  }

  if (qualifier.minApiLevel) {
    directoryName = [...directoryName, `v${qualifier.minApiLevel}`];
  }

  return path.join(config.resDirPath, directoryName.join("-"), fileName);
}