"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generate = generate;

var fse = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var input = _interopRequireWildcard(require("./input"));

var output = _interopRequireWildcard(require("./output"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

const assetName = "AppIcon.appiconset";
const iosIcons = [{
  idiom: "iphone",
  scale: 2,
  size: 20
}, {
  idiom: "iphone",
  scale: 3,
  size: 20
}, {
  idiom: "iphone",
  scale: 2,
  size: 29
}, {
  idiom: "iphone",
  scale: 3,
  size: 29
}, {
  idiom: "iphone",
  scale: 2,
  size: 40
}, {
  idiom: "iphone",
  scale: 3,
  size: 40
}, {
  idiom: "iphone",
  scale: 2,
  size: 60
}, {
  idiom: "iphone",
  scale: 3,
  size: 60
}, {
  idiom: "ipad",
  scale: 1,
  size: 20
}, {
  idiom: "ipad",
  scale: 2,
  size: 20
}, {
  idiom: "ipad",
  scale: 1,
  size: 29
}, {
  idiom: "ipad",
  scale: 2,
  size: 29
}, {
  idiom: "ipad",
  scale: 1,
  size: 40
}, {
  idiom: "ipad",
  scale: 2,
  size: 40
}, {
  idiom: "ipad",
  scale: 1,
  size: 76
}, {
  idiom: "ipad",
  scale: 2,
  size: 76
}, {
  idiom: "ipad",
  scale: 2,
  size: 83.5
}, {
  idiom: "ios-marketing",
  scale: 1,
  size: 1024,
  flattenAlpha: true
}];

async function* generate(config, fileInput) {
  const fullConfig = await getConfig(config);
  yield* generateImages(fullConfig, fileInput);
  yield* generateManifest(fullConfig);
}

async function getConfig(config) {
  return {
    iconsetDir: config.iconsetDir || `./ios/${await getAppName()}/Images.xcassets`
  };
}

async function getAppName() {
  const appJson = JSON.parse((await fse.readFile("./app.json", "utf-8")));
  return appJson.name;
}

async function* generateImages(config, fileInput) {
  yield* output.genaratePngs({ ...fileInput,
    cropSize: input.inputContentSize
  }, iosIcons.map(icon => ({
    filePath: path.join(config.iconsetDir, assetName, getIconFilename(icon)),
    flattenAlpha: icon.flattenAlpha,
    outputSize: icon.size * icon.scale
  })));
}

async function* generateManifest(config) {
  const fileName = path.join(config.iconsetDir, assetName, "Contents.json");
  yield* output.ensureFileContents(fileName, {
    images: iosIcons.map(icon => ({
      filename: getIconFilename(icon),
      idiom: icon.idiom,
      scale: `${icon.scale}x`,
      size: `${icon.size}x${icon.size}`
    })),
    info: {
      author: "react-native-svg-app-icon",
      version: 1
    }
  });
}

function getIconFilename(icon) {
  return `${icon.idiom}-${icon.size}@${icon.scale}x.png`;
}