"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.genaratePngs = genaratePngs;
exports.ensureFileContents = ensureFileContents;

var fse = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var input = _interopRequireWildcard(require("./input"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

async function* genaratePngs(fileInput, outputs) {
  for (let output of outputs) {
    yield* genaratePng(fileInput, output);
  }
}

async function* genaratePng(fileInput, output) {
  if (!(await hasChanged(fileInput, output))) {
    return;
  }

  const {
    sharp,
    imageData
  } = await fileInput.read();
  const metadata = imageData.metadata;
  await fse.ensureDir(path.dirname(output.filePath));
  const scale = fileInput.cropSize === undefined ? 1 : input.inputImageSize / fileInput.cropSize;
  const targetDensity = output.outputSize / metadata.width * metadata.density * scale;
  let image = sharp(imageData.data, {
    density: targetDensity
  });
  const extractRegion = getExtractRegion(targetDensity, metadata, output.outputSize);
  image = image.extract(extractRegion);

  if (fileInput.mask) {
    image = sharp((await image.composite([{
      input: await sharp(Buffer.from(fileInput.mask, "utf-8"), {
        // TODO: Ensure correct density on differing input image sizes
        density: targetDensity
      }).extract(extractRegion).toBuffer(),
      blend: "dest-in"
    }]).toBuffer()));

    if (fileInput.overlay) {
      image = image.composite([{
        input: await sharp(Buffer.from(fileInput.overlay, "utf-8"), {
          // TODO: Ensure correct density on differing input image sizes
          density: targetDensity
        }).extract(extractRegion).toBuffer(),
        blend: "over"
      }]);
    }
  } else {
    image = image.removeAlpha();
  }

  await image.png({
    adaptiveFiltering: false,
    compressionLevel: 9
  }).toFile(output.filePath);
  yield output.filePath;
}

function getExtractRegion(targetDensity, metadata, outputSize) {
  const imageMargin = Math.floor((targetDensity / metadata.density * metadata.width - outputSize) / 2);
  return {
    top: imageMargin,
    left: imageMargin,
    width: outputSize,
    height: outputSize
  };
}

async function hasChanged(input, output) {
  const inputStat = input.fileStats;
  let outputStat;

  try {
    outputStat = await fse.stat(output.filePath);
  } catch {
    return true;
  }

  if (inputStat.mtimeMs > outputStat.mtimeMs) {
    return true;
  } else {
    return false;
  }
}

async function* ensureFileContents(path, content) {
  let stringContent;

  switch (typeof content) {
    case "object":
      stringContent = JSON.stringify(content, undefined, 2);
      break;

    case "string":
      stringContent = content;
      break;

    default:
      throw Error("Invalid content");
  }

  const contentBuffer = Buffer.from(stringContent, "utf-8");
  let equal;

  try {
    const diskFileBuffer = await fse.readFile(path);
    equal = diskFileBuffer.equals(contentBuffer);
  } catch {
    equal = false;
  }

  if (equal) {
    return;
  } else {
    await fse.outputFile(path, contentBuffer);
    yield path;
  }
}